=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2006-02-10 11:03:38 +0000
+++ bzrlib/repository.py	2006-02-16 17:10:15 +0000
@@ -35,6 +35,7 @@
 from bzrlib.tree import RevisionTree
 from bzrlib.testament import Testament
 from bzrlib.tree import EmptyTree
+import bzrlib.ui
 import bzrlib.xml5
 
 
@@ -266,10 +267,12 @@
                 else:
                     # FIXME do not peek!
                     if self.control_files._transport.listable():
+                        pb = bzrlib.ui.ui_factory.progress_bar()
+                        copy_all(self.weave_store, destination.weave_store, pb=pb)
+                        pb.update('copying inventory', 0, 1)
                         destination.control_weaves.copy_multi(self.control_weaves,
                                                               ['inventory'])
-                        copy_all(self.weave_store, destination.weave_store)
-                        copy_all(self.revision_store, destination.revision_store)
+                        copy_all(self.revision_store, destination.revision_store, pb=pb)
                     else:
                         destination.fetch(self, revision_id=revision_id)
             # compatible v4 stores

=== modified file 'bzrlib/store/__init__.py'
--- bzrlib/store/__init__.py	2006-01-03 06:18:04 +0000
+++ bzrlib/store/__init__.py	2006-02-16 17:10:15 +0000
@@ -35,6 +35,7 @@
 from bzrlib.trace import mutter
 import bzrlib.transport as transport
 from bzrlib.transport.local import LocalTransport
+import bzrlib.ui
 
 ######################################################################
 # stores
@@ -101,9 +102,8 @@
             followed by a list of entries which could not be copied (because they
             were missing)
         """
-        if pb is None:
-            pb = bzrlib.ui.ui_factory.progress_bar()
-        pb.update('preparing to copy')
+        if pb:
+            pb.update('preparing to copy')
         failed = set()
         count = 0
         ids = list(ids) # get the list for showing a length.
@@ -118,14 +118,16 @@
                         self._copy_one(fileid, suffix, other, pb)
                     except KeyError:
                         pass
-                pb.update('copy', count, len(ids))
+                if pb:
+                    pb.update('copy', count, len(ids))
             except KeyError:
                 if permit_failure:
                     failed.add(fileid)
                 else:
                     raise
         assert count == len(ids)
-        pb.clear()
+        if pb:
+            pb.clear()
         return count, failed
 
     def _copy_one(self, fileid, suffix, other, pb):
@@ -311,15 +313,23 @@
     return bzrlib.store.text.TextStore(transport.memory.MemoryTransport())
         
 
-def copy_all(store_from, store_to):
+def copy_all(store_from, store_to, pb=None):
     """Copy all ids from one store to another."""
     # TODO: Optional progress indicator
     if not store_from.listable():
         raise UnlistableStore(store_from)
-    ids = [f for f in store_from]
+    ids = []
+    for count, file_id in enumerate(store_from):
+        if pb:
+            pb.update('listing files', count, count)
+        ids.append(file_id)
+    if pb:
+        pb.clear()
     mutter('copy_all ids: %r', ids)
-    store_to.copy_multi(store_from, ids)
+    store_to.copy_multi(store_from, ids, pb=pb)
+
 
 def hash_prefix(fileid):
     return "%02x/" % (adler32(fileid) & 0xff)
 
+

=== modified file 'bzrlib/store/weave.py'
--- bzrlib/store/weave.py	2006-02-10 11:03:38 +0000
+++ bzrlib/store/weave.py	2006-02-16 17:10:15 +0000
@@ -164,8 +164,12 @@
         w.add_identical(old_rev_id, new_rev_id, parent_idxs)
         self.put_weave(file_id, w, transaction)
      
-    def copy_multi(self, from_store, file_ids):
+    def copy_multi(self, from_store, file_ids, pb=None):
         assert isinstance(from_store, WeaveStore)
-        for f in file_ids:
+        for count, f in enumerate(file_ids):
             mutter("copy weave {%s} into %s", f, self)
+            if pb:
+                pb.update('copy', count, len(file_ids))
             self._put(f, from_store._get(f))
+        if pb:
+            pb.clear()

